@@ -3,6 +3,7 @@ import tensorflow as tf
 import tensorflow_addons as tfa
 
 
+@tf.keras.utils.register_keras_serializable()
 class ClassToken(tf.keras.layers.Layer):
     """Append a class token to an input layer."""
 
@@ -23,7 +24,16 @@ class ClassToken(tf.keras.layers.Layer):
         )
         return tf.concat([cls_broadcasted, inputs], 1)
 
+    def get_config(self):
+        config = super().get_config()
+        return config
 
+    @classmethod
+    def from_config(cls, config):
+        return cls(**config)
+
+
+@tf.keras.utils.register_keras_serializable()
 class AddPositionEmbs(tf.keras.layers.Layer):
     """Adds (optionally learned) positional embeddings to the inputs."""
 
@@ -43,7 +53,16 @@ class AddPositionEmbs(tf.keras.layers.Layer):
     def call(self, inputs):
         return inputs + tf.cast(self.pe, dtype=inputs.dtype)
 
+    def get_config(self):
+        config = super().get_config()
+        return config
 
+    @classmethod
+    def from_config(cls, config):
+        return cls(**config)
+
+
+@tf.keras.utils.register_keras_serializable()
 class MultiHeadSelfAttention(tf.keras.layers.Layer):
     def __init__(self, *args, num_heads, **kwargs):
         super().__init__(*args, **kwargs)
@@ -91,7 +110,18 @@ class MultiHeadSelfAttention(tf.keras.layers.Layer):
         output = self.combine_heads(concat_attention)
         return output, weights
 
+    def get_config(self):
+        config = super().get_config()
+        config.update({"num_heads": self.num_heads})
+        return config
 
+    @classmethod
+    def from_config(cls, config):
+        return cls(**config)
+
+
+# pylint: disable=too-many-instance-attributes
+@tf.keras.utils.register_keras_serializable()
 class TransformerBlock(tf.keras.layers.Layer):
     """Implements a Transformer block."""
 
@@ -132,13 +162,28 @@ class TransformerBlock(tf.keras.layers.Layer):
         self.layernorm2 = tf.keras.layers.LayerNormalization(
             epsilon=1e-6, name="LayerNorm_2"
         )
-        self.dropout = tf.keras.layers.Dropout(self.dropout)
+        self.dropout_layer = tf.keras.layers.Dropout(self.dropout)
 
     def call(self, inputs, training):
         x = self.layernorm1(inputs)
         x, weights = self.att(x)
-        x = self.dropout(x, training=training)
+        x = self.dropout_layer(x, training=training)
         x = x + inputs
         y = self.layernorm2(x)
         y = self.mlpblock(y)
         return x + y, weights
+
+    def get_config(self):
+        config = super().get_config()
+        config.update(
+            {
+                "num_heads": self.num_heads,
+                "mlp_dim": self.mlp_dim,
+                "dropout": self.dropout,
+            }
+        )
+        return config
+
+    @classmethod
+    def from_config(cls, config):
+        return cls(**config)
